@@ -3,11 +3,11 @@ import logging
 from urllib.parse import urlparse
 from async_timeout import timeout
 import serial
-from serial import SerialException
 from serial_asyncio import create_serial_connection
-from .vpackets import *
 from scapy.packet import Raw
 
+from .vpackets import *
+
 logger = logging.getLogger(__name__)
 
 class ViessmannProtocol(asyncio.Protocol):
@@ -44,8 +44,9 @@ class ViessmannProtocol(asyncio.Protocol):
                 self._buf = self._buf[1:]
                 if self._running:
                     if not self._viessmann_lock.locked():
-                        logger.debug("Viessmann disconnected")
-                        self._connected = False
+                        if self._connected:
+                            logger.debug("Viessmann disconnected")
+                            self._connected = False
                     else:
                         self._periodic_event.set()
             elif self._buf[0] == self._START_BYTE:
@@ -71,7 +72,7 @@ class ViessmannProtocol(asyncio.Protocol):
                 if pkt.answers(self._sent_pkt):
                     self._result_future.set_result(pkt)
                 else:
-                    logger.debug("packet is not answering")
+                    logger.error("%s does not answer %s", pkt.show(dump=True), self._sent_pkt.show(dump=True))
                     self._result_future.cancel()
             else:
                 logger.error("Unknown byte: %02x" % (self._buf[0]))
@@ -84,19 +85,31 @@ class ViessmannProtocol(asyncio.Protocol):
         await asyncio.wait_for(self._ack_event.wait(), timeout)
         self._ack_event.clear()
 
-    async def send_recv(self, pkt, timeout=3):
+    async def send(self, pkt, timeout=3):
         if not self._connected:
             logger.debug("Reconnecting...")
             await self._reconnect_viessmann()
+        await self._send_ack(bytes(pkt), timeout)
+
+    async def send_recv(self, pkt, timeout=3):
         self._sent_pkt = pkt
         self._result_future = self._loop.create_future()
-        await self._send_ack(bytes(pkt), timeout)
-        logger.debug("Request sent successfuly, waiting response...")
+        await self.send(pkt, timeout)
         answer = await asyncio.wait_for(self._result_future, timeout)
-        logger.debug("Response received:")
-        logger.debug(answer.show(dump=True))
         return answer
 
+    async def readdata(self, address, length):
+        request = VS2Header() / VS2Data(type="request", command="readdata", address=address, data_len=length)
+        response = await self.send_recv(request)
+        if response.type == 0x01: # response
+            return response.data
+        else:
+            raise Exception("error reading data: %s" % (response.show(dump=True)))
+
+    async def writedata(self, address, data):
+        equest = VS2Header() / VS2Data(type="request", command="writedata", address=address, data_len=len(data), data=data)
+        await self.send(request)
+
     async def _reconnect_viessmann(self):
         self._periodic_event.clear()
         async with self._viessmann_lock:
@@ -136,7 +149,7 @@ class ViessmannProtocol(asyncio.Protocol):
                 async with timeout(5, loop=self._loop):
                     self._transport, _ = await self._create_connection()
             except (BrokenPipeError, ConnectionRefusedError,
-                    SerialException, asyncio.TimeoutError) as exc:
+                    serial.SerialException, asyncio.TimeoutError) as exc:
                 logger.warning(exc)
                 asyncio.ensure_future(self._reconnect(), loop=self._loop)
             else:
